@@ -1,9 +1,12 @@
 from wcs.services.mgrbase import MgrBase
-from wcs.commons.config import MGR_URL
+from wcs.commons.config import Config
 from wcs.commons.http import _post
 from wcs.commons.http import _get
-from wcs.commons.util import urlsafe_base64_encode
-from wcs.commons.logme import debug, warning, error
+from wcs.commons.util import urlsafe_base64_encode,https_check
+
+from wcs.commons.logme import debug, error
+
+MGR_URL = Config.mgr_url
 
 class PersistentFop(MgrBase):
 
@@ -34,16 +37,14 @@ class PersistentFop(MgrBase):
     def fops_status(self, persistentId):
         url = '{0}/status/get/prefop?persistentId={1}'.format(self.mgr_host, persistentId)
         debug('Start to get status of persistentId: %s' % persistentId)
-        code, text = _get(url=url)
-        debug('The return code and text of status request is:%d %s' % (code, text))
-        return code, text
+        return _get(url=url)
 
     def _gernerate_headers(self,url,data):
         reqdata = super(PersistentFop, self)._params_parse(data)
         headers = super(PersistentFop, self)._gernerate_headers(url, body=reqdata) 
         return headers,reqdata
 
-    def execute(self,fops,bucket,key,force=None,separate=None,notifyurl=None):
+    def execute(self,fops,bucket,key,force=0,separate=0,notifyurl=None):
         data = {'bucket': urlsafe_base64_encode(bucket), 'key': urlsafe_base64_encode(key), 'fops': urlsafe_base64_encode(fops)}
         if notifyurl is not None:
             data['notifyURL'] = urlsafe_base64_encode(notifyurl)
@@ -51,12 +52,10 @@ class PersistentFop(MgrBase):
             data['force'] = 1
         if separate == 1:
             data['separate'] = 1
-        url = '{0}/fops'.format(self.mgr_host)
+        url = https_check('{0}/fops'.format(self.mgr_host))
         headers,reqdata = self._gernerate_headers(url,data)
         debug('PersistentFops is %s' % fops)
         debug('Start to post persistentFops')
-        code,text = _post(url=url, data=reqdata,headers=headers)
-        debug('Code: %d, text: %s' % (code, text))
-        return code,text
+        return _post(url=url, data=reqdata,headers=headers)
 
 
