@@ -2,7 +2,8 @@ import os
 import requests
 from requests_toolbelt import MultipartEncoder
 from wcs.commons.http import _post
-from wcs.commons.logme import debug, warning, error
+from wcs.commons.logme import debug,error
+from wcs.commons.util import https_check
 
 class SimpleUpload(object):
 
@@ -15,13 +16,14 @@ class SimpleUpload(object):
         fileds['file'] = ('filename', f, 'text/plain')
         encoder = MultipartEncoder(fileds)
         headers = {"Content-Type":encoder.content_type}
-        headers['user-agent'] = "WCS-Python-SDK-3.0.2(http://wcs.chinanetcenter.com)"
+        headers['user-agent'] = "WCS-Python-SDK-4.0.0(http://wcs.chinanetcenter.com)"
         return url, encoder, headers  
     
     def _gernerate_content(self,path):
         return open(path, 'rb')
 
     def _upload(self,url,encoder,headers,f):
+        url = https_check(url)
         try:
             r = requests.post(url=url, headers=headers, data=encoder, verify=True)
         except Exception as e:
@@ -33,10 +35,11 @@ class SimpleUpload(object):
             debug(e)
             return -1,e
         f.close()
-        debug('Status Code:' + str(r.status_code))
-        debug('Response Body:')
-        debug(r.text)
-        return r.status_code,r.text
+        try:
+            r_header = {'x-reqid': r.headers['x-reqid']}
+            return r.status_code,r.text,r_headers
+        except:           
+            return r.status_code,r.text
 
     def upload(self, filepath,token):
         if os.path.exists(filepath) and os.path.isfile(filepath):
